import Handlebars from 'handlebars'
import juice from 'juice'
import * as utils from './utils'
import { FOUNDATION, BASE_TEMPLATE } from './constants'

/**
 * Creates a template compiler
 * @param {CompilerParams} globalParams
 */
function Compiler (globalParams = {}) {
  const hbs = Handlebars.create()

  const globalCss = [FOUNDATION, ...utils.coerceArray(globalParams.css)]
  const BaseTemplate = hbs.compile(globalParams.baseTemplate || BASE_TEMPLATE)

  const { helpers = {}, partials = {} } = globalParams
  utils.registerHelpers(hbs, helpers)
  utils.registerPartials(hbs, partials)

  /**
   * Compiles template
   * @param {CompilerInstanceParams} params
   */
  function compile (params) {
    const css = [...globalCss, ...utils.coerceArray(params.css)]
    const template = hbs.compile(
      BaseTemplate({ body: params.template, css })
    )

    return utils.compose(
      juice,
      utils.inkify,
      data => template(data),
      data => globalParams.data
        ? { ...globalParams.data, ...data }
        : data
    )
  }

  return compile
}

export default Compiler

/**
 * @typedef {object} CompilerParams
 * @property {object} data global template data
 * @property {object} partials handlebars partials
 * @property {object} helpers handlebars helpers
 * @property {string} baseTemplate base template
 * @property {string|string[]} css global style string(s)
 */

/**
 * @typedef {object} CompilerInstanceParams
 * @property {string} template template string
 * @property {string|string[]} css local style string(s)
 */
