import fs from 'fs'
import { Inky } from 'inky'
import promisify from 'util-promisify'

const inky = new Inky()
export const readFile = promisify(fs.readFile)
readFile.sync = fs.readFileSync

export const coerceArray = value => value === undefined
  ? []
  : Array.isArray(value)
    ? value
    : [value]

export const compose = (...fns) => arg => fns.reduceRight(
  (arg, fn) => fn(arg),
  arg
)

export const trace = (tag = '', transform = a => a) =>
  value => console.log(tag, transform(value)) || value

export const eachObj = (fn, obj) => Object.keys(obj)
  .forEach(key => fn(key, obj[key], obj))

export const replace = (re, substr) => str => str.replace(re, substr)

export const inkify = html => inky
  .releaseTheKraken(html, { decodeEntities: false })
  .replace(/\{\{&gt;/g, '{{>')
  .replace(/\{\{#&gt;/g, '{{#>')
  .replace(/&apos;/g, '\'')

export const registerPartials = (hbs, partials) => eachObj(
  (name, partial) => hbs.registerPartial(name, inkify(partial)),
  partials
)

export const registerHelpers = (hbs, helpers) => eachObj(
  (name, helper) => hbs.registerHelper(name, helper),
  helpers
)
